@@ -1,13 +1,13 @@
 /*
-  $Id: check_user.c,v 1.2 2000/12/04 19:02:33 baggins Exp $
-  
+  $Id$
+
   This program was contributed by Shane Watts <shane@icarus.bofh.asn.au>
   slight modifications by AGM.
 
   You need to add the following (or equivalent) to the /etc/pam.conf file.
   # check authorization
-  check   auth       required     pam_unix_auth.so
-  check   account    required     pam_unix_acct.so
+  check   auth       required     pam_unix.so
+  check   account    required     pam_unix.so
 */
 
 #include <security/pam_appl.h>
@@ -35,7 +35,7 @@ int main(int argc, char *argv[])
     }
 
     retval = pam_start("check", user, &conv, &pamh);
-	
+
     if (retval == PAM_SUCCESS)
         retval = pam_authenticate(pamh, 0);    /* is user really user? */
 
