@@ -8,6 +8,7 @@
  */
 
 #include "pam_modutil_private.h"
+#include "pam_inline.h"
 
 #include <errno.h>
 #include <limits.h>
@@ -15,26 +16,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-static int intlen(int number)
-{
-    int len = 2;
-    while (number != 0) {
-        number /= 10;
-	len++;
-    }
-    return len;
-}
-
-static int longlen(long number)
-{
-    int len = 2;
-    while (number != 0) {
-        number /= 10;
-	len++;
-    }
-    return len;
-}
-
 struct passwd *
 pam_modutil_getpwuid(pam_handle_t *pamh, uid_t uid)
 {
@@ -65,28 +46,25 @@ pam_modutil_getpwuid(pam_handle_t *pamh, uid_t uid)
 			    sizeof(struct passwd) + (char *) buffer,
 			    length, &result);
 	if (!status && (result == buffer)) {
-	    char *data_name;
 	    const void *ignore;
 	    int i;
 
-	    data_name = malloc(strlen("_pammodutil_getpwuid") + 1 +
-			       longlen((long) uid) + 1 + intlen(INT_MAX) + 1);
-	    if ((pamh != NULL) && (data_name == NULL)) {
-	        D(("was unable to register the data item [%s]",
-	           pam_strerror(pamh, status)));
-		free(buffer);
-		return NULL;
-	    }
-
 	    if (pamh != NULL) {
 	        for (i = 0; i < INT_MAX; i++) {
-	            sprintf(data_name, "_pammodutil_getpwuid_%ld_%d",
-			    (long) uid, i);
+		    char *data_name = pam_asprintf("_pammodutil_getpwuid_%ld_%d",
+						   (long) uid, i);
+		    if (data_name == NULL) {
+			D(("was unable to register the data item [%s]",
+			   pam_strerror(pamh, status)));
+			free(buffer);
+			return NULL;
+		    }
 		    status = PAM_NO_MODULE_DATA;
 	            if (pam_get_data(pamh, data_name, &ignore) != PAM_SUCCESS) {
 		        status = pam_set_data(pamh, data_name,
-					      result, pam_modutil_cleanup);
+					      result, pam_modutil_cleanup_passwd);
 		    }
+		    free(data_name);
 		    if (status == PAM_SUCCESS) {
 		        break;
 		    }
@@ -95,8 +73,6 @@ pam_modutil_getpwuid(pam_handle_t *pamh, uid_t uid)
 	        status = PAM_SUCCESS;
 	    }
 
-	    free(data_name);
-
 	    if (status == PAM_SUCCESS) {
 		D(("success"));
 		return result;
