@@ -1,5 +1,5 @@
 /*
- * $Id: libpamc.h,v 1.2 2000/11/19 23:54:03 agmorgan Exp $
+ * $Id$
  *
  * Copyright (c) Andrew G. Morgan <morgan@ftp.kernel.org>
  *
@@ -8,6 +8,8 @@
 #ifndef LIBPAMC_H
 #define LIBPAMC_H
 
+#include "config.h"
+
 #include <security/pam_client.h>
 #include <security/_pam_macros.h>
 
@@ -16,6 +18,7 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <sys/wait.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <ctype.h>
