@@ -141,26 +141,11 @@ static char *getNISserver(pam_handle_t *pamh, unsigned long long ctrl)
 #endif
 
 
-#ifdef HAVE_YP_GET_DEFAULT_DOMAIN
 	if ((err = yp_get_default_domain(&domainname)) != 0) {
 		pam_syslog(pamh, LOG_WARNING, "can't get local yp domain: %s",
 			 yperr_string(err));
 		return NULL;
 	}
-#elif defined(HAVE_GETDOMAINNAME)
-	char domainname_res[256];
-
-	if (getdomainname (domainname_res, sizeof (domainname_res)) == 0)
-	  {
-	    if (strcmp (domainname_res, "(none)") == 0)
-	      {
-		/* If domainname is not set, some systems will return "(none)" */
-		domainname_res[0] = '\0';
-	      }
-	    domainname = domainname_res;
-	  }
-	else domainname = NULL;
-#endif
 
 	if ((err = yp_master(domainname, "passwd.byname", &master)) != 0) {
 		pam_syslog(pamh, LOG_WARNING, "can't find the master ypserver: %s",
@@ -510,7 +495,7 @@ static int _unix_verify_shadow(pam_handle_t *pamh, const char *user, unsigned lo
 {
 	struct passwd *pwent = NULL;	/* Password and shadow password */
 	struct spwd *spent = NULL;	/* file entries for the user */
-	int daysleft;
+	long daysleft;
 	int retval;
 
 	retval = get_account_info(pamh, user, &pwent, &spent);
@@ -529,6 +514,10 @@ static int _unix_verify_shadow(pam_handle_t *pamh, const char *user, unsigned lo
 	else if (retval == PAM_SUCCESS)
 		retval = check_shadow_expiry(pamh, spent, &daysleft);
 
+	if (on(UNIX_EXPIRED_ONLY, ctrl)) {
+		return retval == PAM_NEW_AUTHTOK_REQD ? PAM_SUCCESS : PAM_AUTHTOK_ERR;
+	}
+
 	if (on(UNIX__IAMROOT, ctrl) || retval == PAM_NEW_AUTHTOK_REQD)
 		return PAM_SUCCESS;
 
@@ -709,7 +698,10 @@ pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const char **argv)
 		pass_old = NULL;
 		retval = _unix_verify_shadow(pamh,user, ctrl);
 		if (retval == PAM_AUTHTOK_ERR) {
-			if (off(UNIX__IAMROOT, ctrl))
+			if (on(UNIX_EXPIRED_ONLY, ctrl))
+				_make_remark(pamh, ctrl, PAM_ERROR_MSG,
+					     _("Your password is not expired."));
+			else if (off(UNIX__IAMROOT, ctrl))
 				_make_remark(pamh, ctrl, PAM_ERROR_MSG,
 					     _("You must wait longer to change your password."));
 			else
