@@ -23,6 +23,9 @@
 #ifdef HAVE_CRYPT_H
 #include <crypt.h>
 #endif
+#ifdef USE_PWACCESS
+#include <pwaccess.h>
+#endif
 
 #include "pam_cc_compat.h"
 #include "pam_inline.h"
@@ -94,7 +97,7 @@ PAMH_ARG_DECL(int verify_pwd_hash,
 	} else {
 		if (pam_str_skip_prefix(hash, "$1$") != NULL) {
 			pp = Goodcrypt_md5(p, hash);
-			if (pp && strcmp(pp, hash) != 0) {
+			if (pp && !pam_consttime_streq(pp, hash)) {
 				_pam_delete(pp);
 				pp = Brokencrypt_md5(p, hash);
 			}
@@ -163,7 +166,7 @@ PAMH_ARG_DECL(int verify_pwd_hash,
 		/* the moment of truth -- do we agree with the password? */
 		D(("comparing state of pp[%s] and hash[%s]", pp ? pp : "(null)", hash));
 
-		if (pp && strcmp(pp, hash) == 0) {
+		if (pp && pam_consttime_streq(pp, hash)) {
 			retval = PAM_SUCCESS;
 		} else {
 			retval = PAM_AUTH_ERR;
@@ -200,66 +203,52 @@ PAMH_ARG_DECL(int get_account_info,
 	*pwd = pam_modutil_getpwnam(pamh, name);	/* Get password file entry... */
 	*spwdent = NULL;
 
-	if (*pwd != NULL) {
-		if (strcmp((*pwd)->pw_passwd, "*NP*") == 0)
-		{ /* NIS+ */
-#ifdef HELPER_COMPILE
-			uid_t save_euid, save_uid;
-
-			save_euid = geteuid();
-			save_uid = getuid();
-			if (save_uid == (*pwd)->pw_uid) {
-				if (setreuid(save_euid, save_uid))
-					return PAM_CRED_INSUFFICIENT;
-			} else  {
-				if (setreuid(0, -1))
-					return PAM_CRED_INSUFFICIENT;
-				if (setreuid(-1, (*pwd)->pw_uid)) {
-					if (setreuid(-1, 0)
-					    || setreuid(0, -1)
-					    || setreuid(-1, (*pwd)->pw_uid)) {
-						return PAM_CRED_INSUFFICIENT;
-					}
-				}
-			}
-
-			*spwdent = pam_modutil_getspnam(pamh, name);
-			if (save_uid == (*pwd)->pw_uid) {
-				if (setreuid(save_uid, save_euid))
-					return PAM_CRED_INSUFFICIENT;
-			} else {
-				if (setreuid(-1, 0)
-				    || setreuid(save_uid, -1)
-				    || setreuid(-1, save_euid))
-					return PAM_CRED_INSUFFICIENT;
-			}
-
-			if (*spwdent == NULL || (*spwdent)->sp_pwdp == NULL)
-				return PAM_AUTHINFO_UNAVAIL;
-#else
-			/* we must run helper for NIS+ passwords */
-			return PAM_UNIX_RUN_HELPER;
-#endif
-		} else if (is_pwd_shadowed(*pwd)) {
-#ifdef HELPER_COMPILE
-			/*
-			 * shadow password file entry for this user,
-			 * if shadowing is enabled
-			 */
-			*spwdent = getspnam(name);
-			if (*spwdent == NULL || (*spwdent)->sp_pwdp == NULL)
-				return PAM_AUTHINFO_UNAVAIL;
-#else
-			/*
-			 * The helper has to be invoked to deal with
-			 * the shadow password file entry.
-			 */
-			return PAM_UNIX_RUN_HELPER;
-#endif
-		}
-	} else {
+	if (*pwd == NULL) {
 		return PAM_USER_UNKNOWN;
 	}
+
+	if (is_pwd_shadowed(*pwd)) {
+#if defined(HELPER_COMPILE) || defined(PAM_UNIX_TRY_GETSPNAM)
+		/*
+		 * shadow password file entry for this user,
+		 * if shadowing is enabled
+		 */
+#ifdef USE_PWACCESS
+		int r;
+		bool complete = false;
+		char *error = NULL;
+
+		r = pwaccess_get_user_record(-1, name, NULL, spwdent, &complete, &error);
+		if (r < 0) {
+			if (!PWACCESS_IS_NOT_RUNNING(r))
+				pam_syslog(pamh, LOG_ERR, "%s",
+					   error ? error : strerror(-r));
+			free(error);
+		}
+		if (complete)
+			return PAM_SUCCESS;
+		struct_shadow_freep(spwdent);
+#endif
+		*spwdent = pam_modutil_getspnam(pamh, name);
+		if (*spwdent == NULL || (*spwdent)->sp_pwdp == NULL
+# ifndef HELPER_COMPILE
+			/* synthesized entry from libnss-systemd */
+			|| (strcmp(name, "root") == 0 &&
+			    strcmp((*spwdent)->sp_pwdp, "!*") == 0)
+# endif
+		   )
+# ifdef HELPER_COMPILE
+			return PAM_AUTHINFO_UNAVAIL;
+# endif
+#endif /* HELPER_COMPILE || PAM_UNIX_TRY_GETSPNAM */
+#ifndef HELPER_COMPILE
+		/*
+		 * The helper has to be invoked to deal with
+		 * the shadow password file entry.
+		 */
+		return PAM_UNIX_RUN_HELPER;
+#endif
+	}
 	return PAM_SUCCESS;
 }
 
@@ -284,23 +273,8 @@ PAMH_ARG_DECL(int get_pwd_hash,
 	return PAM_SUCCESS;
 }
 
-/*
- * invariant: 0 <= num1
- * invariant: 0 <= num2
- */
-static int
-subtract(long num1, long num2)
-{
-	long value = num1 - num2;
-	if (value < INT_MIN)
-		return INT_MIN;
-	if (value > INT_MAX)
-		return INT_MAX;
-	return (int)value;
-}
-
 PAMH_ARG_DECL(int check_shadow_expiry,
-	struct spwd *spent, int *daysleft)
+	struct spwd *spent, long *daysleft)
 {
 	long int curdays, passed;
 	*daysleft = -1;
@@ -331,7 +305,7 @@ PAMH_ARG_DECL(int check_shadow_expiry,
 			long inact = spent->sp_max < LONG_MAX - spent->sp_inact ?
 			    spent->sp_max + spent->sp_inact : LONG_MAX;
 			if (passed >= inact) {
-				*daysleft = subtract(inact, passed);
+				*daysleft = inact - passed;
 				D(("authtok expired"));
 				return PAM_AUTHTOK_EXPIRED;
 			}
@@ -344,7 +318,7 @@ PAMH_ARG_DECL(int check_shadow_expiry,
 			long warn = spent->sp_warn > spent->sp_max ? -1 :
 			    spent->sp_max - spent->sp_warn;
 			if (passed >= warn) {
-				*daysleft = subtract(spent->sp_max, passed);
+				*daysleft = spent->sp_max - passed;
 				D(("warn before expiry"));
 			}
 		}
@@ -362,9 +336,31 @@ PAMH_ARG_DECL(int check_shadow_expiry,
 
 /* passwd/salt conversion macros */
 
-#define PW_TMPFILE              "/etc/npasswd"
-#define SH_TMPFILE              "/etc/nshadow"
-#define OPW_TMPFILE             SCONFIGDIR "/nopasswd"
+#define PW_TMPFILE              "/etc/.pam.passwdXXXXXX"
+#define SH_TMPFILE              "/etc/.pam.shadowXXXXXX"
+#define OPW_TMPFILE             SCONFIG_DIR "/.pam.opasswdXXXXXX"
+
+/*
+ * fmkstemp - creates and opens a temporary file from given pattern
+ */
+static FILE *
+fmkstemp(char *name)
+{
+        FILE *fp;
+        int fd;
+
+        fd = mkstemp(name);
+        if (fd == -1)
+                return NULL;
+
+        fp = fdopen(fd, "we");
+        if (fp == NULL) {
+                close(fd);
+                return NULL;
+        }
+
+        return fp;
+}
 
 /*
  * i64c - convert an integer to a radix 64 character
@@ -522,20 +518,20 @@ PAMH_ARG_DECL(char * create_password_hash,
 			   on(UNIX_BLOWFISH_PASS, ctrl) ? "blowfish" :
 			   on(UNIX_SHA256_PASS, ctrl) ? "sha256" :
 			   on(UNIX_SHA512_PASS, ctrl) ? "sha512" : algoid);
-		if(sp) {
-		   pam_overwrite_string(sp);
-		}
 #ifdef HAVE_CRYPT_R
 		pam_overwrite_object(cdata);
 		free(cdata);
+#else
+		pam_overwrite_string(sp);
 #endif
 		return NULL;
 	}
 	ret = strdup(sp);
-	pam_overwrite_string(sp);
 #ifdef HAVE_CRYPT_R
 	pam_overwrite_object(cdata);
 	free(cdata);
+#else
+	pam_overwrite_string(sp);
 #endif
 	return ret;
 }
@@ -653,12 +649,12 @@ save_old_password(pam_handle_t *pamh, const char *forwho, const char *oldpass,
 		  int howmany)
 #endif
 {
+    char tempfile[] = OPW_TMPFILE;
     char *buf = NULL;
     char *s_luser, *s_uid, *s_npas, *s_pas, *pass;
     int npas;
     FILE *pwfile, *opwfile;
     int err = 0;
-    mode_t oldmask;
     int found = 0;
     struct passwd *pwd = NULL;
     struct stat st;
@@ -676,8 +672,6 @@ save_old_password(pam_handle_t *pamh, const char *forwho, const char *oldpass,
 	return PAM_SUCCESS;
     }
 
-    oldmask = umask(077);
-
 #ifdef WITH_SELINUX
     if (SELINUX_ENABLED) {
       char *passwd_context_raw = NULL;
@@ -696,11 +690,10 @@ save_old_password(pam_handle_t *pamh, const char *forwho, const char *oldpass,
       freecon(passwd_context_raw);
     }
 #endif
-    pwfile = fopen(OPW_TMPFILE, "we");
-    umask(oldmask);
+    pwfile = fmkstemp(tempfile);
     if (pwfile == NULL) {
       err = 1;
-      goto done;
+      goto tmp_error;
     }
 
     opwfile = fopen(OLD_PASSWORDS_FILE, "re");
@@ -808,9 +801,12 @@ save_old_password(pam_handle_t *pamh, const char *forwho, const char *oldpass,
 
 done:
     if (!err) {
-	if (rename(OPW_TMPFILE, OLD_PASSWORDS_FILE))
+	if (rename(tempfile, OLD_PASSWORDS_FILE))
 	    err = 1;
     }
+    if (err)
+	unlink(tempfile);
+tmp_error:
 #ifdef WITH_SELINUX
     if (SELINUX_ENABLED) {
       if (setfscreatecon_raw(prev_context_raw)) {
@@ -821,27 +817,21 @@ done:
       prev_context_raw = NULL;
     }
 #endif
-    if (!err) {
-	return PAM_SUCCESS;
-    } else {
-	unlink(OPW_TMPFILE);
-	return PAM_AUTHTOK_ERR;
-    }
+    return err ? PAM_AUTHTOK_ERR : PAM_SUCCESS;
 }
 
 PAMH_ARG_DECL(int unix_update_passwd,
 	const char *forwho, const char *towhat)
 {
+    char tempfile[] = PW_TMPFILE;
     struct passwd *tmpent = NULL;
     struct stat st;
     FILE *pwfile, *opwfile;
     int err = 1;
-    mode_t oldmask;
 #ifdef WITH_SELINUX
     char *prev_context_raw = NULL;
 #endif
 
-    oldmask = umask(077);
 #ifdef WITH_SELINUX
     if (SELINUX_ENABLED) {
       char *passwd_context_raw = NULL;
@@ -860,11 +850,10 @@ PAMH_ARG_DECL(int unix_update_passwd,
       freecon(passwd_context_raw);
     }
 #endif
-    pwfile = fopen(PW_TMPFILE, "we");
-    umask(oldmask);
+    pwfile = fmkstemp(tempfile);
     if (pwfile == NULL) {
       err = 1;
-      goto done;
+      goto tmp_error;
     }
 
     opwfile = fopen("/etc/passwd", "re");
@@ -928,12 +917,15 @@ PAMH_ARG_DECL(int unix_update_passwd,
 
 done:
     if (!err) {
-	if (!rename(PW_TMPFILE, "/etc/passwd"))
+	if (!rename(tempfile, "/etc/passwd"))
 	    pam_syslog(pamh,
 		LOG_NOTICE, "password changed for %s", forwho);
 	else
 	    err = 1;
     }
+    if (err)
+        unlink(tempfile);
+tmp_error:
 #ifdef WITH_SELINUX
     if (SELINUX_ENABLED) {
       if (setfscreatecon_raw(prev_context_raw)) {
@@ -944,29 +936,22 @@ done:
       prev_context_raw = NULL;
     }
 #endif
-    if (!err) {
-	return PAM_SUCCESS;
-    } else {
-	unlink(PW_TMPFILE);
-	return PAM_AUTHTOK_ERR;
-    }
+    return err ? PAM_AUTHTOK_ERR : PAM_SUCCESS;
 }
 
 PAMH_ARG_DECL(int unix_update_shadow,
 	const char *forwho, char *towhat)
 {
+    char tempfile[] = SH_TMPFILE;
     struct spwd spwdent, *stmpent = NULL;
     struct stat st;
     FILE *pwfile, *opwfile;
     int err = 0;
-    mode_t oldmask;
     int wroteentry = 0;
 #ifdef WITH_SELINUX
     char *prev_context_raw = NULL;
 #endif
 
-    oldmask = umask(077);
-
 #ifdef WITH_SELINUX
     if (SELINUX_ENABLED) {
       char *shadow_context_raw = NULL;
@@ -985,11 +970,10 @@ PAMH_ARG_DECL(int unix_update_shadow,
       freecon(shadow_context_raw);
     }
 #endif
-    pwfile = fopen(SH_TMPFILE, "we");
-    umask(oldmask);
+    pwfile = fmkstemp(tempfile);
     if (pwfile == NULL) {
 	err = 1;
-	goto done;
+	goto tmp_error;
     }
 
     opwfile = fopen("/etc/shadow", "re");
@@ -1073,13 +1057,15 @@ PAMH_ARG_DECL(int unix_update_shadow,
 
  done:
     if (!err) {
-	if (!rename(SH_TMPFILE, "/etc/shadow"))
+	if (!rename(tempfile, "/etc/shadow"))
 	    pam_syslog(pamh,
 		LOG_NOTICE, "password changed for %s", forwho);
 	else
 	    err = 1;
     }
-
+    if (err)
+        unlink(tempfile);
+tmp_error:
 #ifdef WITH_SELINUX
     if (SELINUX_ENABLED) {
       if (setfscreatecon_raw(prev_context_raw)) {
@@ -1090,13 +1076,7 @@ PAMH_ARG_DECL(int unix_update_shadow,
       prev_context_raw = NULL;
     }
 #endif
-
-    if (!err) {
-	return PAM_SUCCESS;
-    } else {
-	unlink(SH_TMPFILE);
-	return PAM_AUTHTOK_ERR;
-    }
+    return err ? PAM_AUTHTOK_ERR : PAM_SUCCESS;
 }
 
 #ifdef HELPER_COMPILE
