@@ -80,6 +80,10 @@ unsigned long long _set_ctrl(pam_handle_t *pamh, int flags, int *remember,
 		D(("IAMROOT"));
 		set(UNIX__IAMROOT, ctrl);
 	}
+	if (flags & PAM_CHANGE_EXPIRED_AUTHTOK) {
+		D(("PAM_CHANGE_EXPIRED_AUTHTOK"));
+		set(UNIX_EXPIRED_ONLY, ctrl);
+	}
 	if (flags & PAM_UPDATE_AUTHTOK) {
 		D(("UPDATE_AUTHTOK"));
 		set(UNIX__UPDATE, ctrl);
@@ -380,7 +384,7 @@ int _unix_getpwnam(pam_handle_t *pamh, const char *name,
 		}
 	}
 
-#if defined(HAVE_NIS) && defined(HAVE_YP_GET_DEFAULT_DOMAIN) && defined (HAVE_YP_BIND) && defined (HAVE_YP_MATCH) && defined (HAVE_YP_UNBIND)
+#if defined(HAVE_NIS)
 	if (!matched && nis) {
 		char *userinfo = NULL, *domain = NULL;
 		int len = 0, i;
@@ -648,7 +652,7 @@ _unix_blankpasswd (pam_handle_t *pamh, unsigned long long ctrl, const char *name
 {
 	struct passwd *pwd = NULL;
 	char *salt = NULL;
-	int daysleft;
+	long daysleft;
 	int retval;
 	int blank = 0;
 	int execloop;
@@ -863,7 +867,7 @@ int
 _unix_verify_user(pam_handle_t *pamh,
                   unsigned long long ctrl,
                   const char *name,
-                  int *daysleft)
+                  long *daysleft)
 {
     int retval;
     struct spwd *spent;
